// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/20AO8C500K/driver/main.h $
// $Rev: 53736 $
// $Date: 2023-09-21 16:09:42 -0500 (Thu, 21 Sep 2023) $

// 20AO8C500K: Device Driver: header file

#ifndef	__MAIN_H__
#define	__MAIN_H__

#include "20ao8c500k.h"

#define	DEV_BAR_SHOW						0
#define	DEV_METRICS_SHOW					0
#define	DEV_PCI_ID_SHOW						0
#define	DEV_SUPPORTS_IRQ
#define	DEV_SUPPORTS_WAIT
#define	DEV_SUPPORTS_WRITE
#define	GSC_ALT_DEV_GET(a)					(a)
#define	GSC_ALT_STRUCT_GET(i,d)				(d)
#define	GSC_ALT_STRUCT_T					dev_data_t
#define	GSC_DEVS_PER_BOARD					1

typedef struct _dev_data_t					dev_data_t;

#include "gsc_main.h"



// macros *********************************************************************

#define	DEV_MODEL							"20AO8C500K"	// Upper case form of the below.
#define	DEV_NAME							"20ao8c500k"	// MUST AGREE WITH AO8C500K_BASE_NAME

#define	DEV_VERSION							"1.0"	// FOR DEVICE SPECIFIC CODE ONLY!
// 1.0	Initial release.

// I/O services
#define	DEV_IO_STREAM_QTY					(DEV_IO_RX_STREAM_QTY + DEV_IO_TX_STREAM_QTY)
#define	DEV_IO_RX_STREAM_QTY				0
#define	DEV_IO_TX_STREAM_QTY				1

#define	GSC_WRITE_PIO_WORK_32_BIT

// WAIT services
#define	DEV_WAIT_GSC_ALL					AO8C500K_WAIT_GSC_ALL
#define	DEV_WAIT_ALT_ALL					AO8C500K_WAIT_ALT_ALL
#define	DEV_WAIT_IO_ALL						AO8C500K_WAIT_IO_ALL



// data types *****************************************************************

struct _dev_io_t
{
	// Initialized by open and IOCTL services (initialize and/or service specific).

	s32					dma_threshold;		// Minimum DMA size, if request not yet met.
	s32					io_mode;			// PIO, BMDMA, DMDMA
	s32					over_data;			// Check data overflow when writing?
	s32					over_frame;			// Check frame overflow when writing?
	s32					pio_threshold;		// Use PIO if samples <= this.
	s32					timeout_s;			// I/O timeout in seconds.

	// Initialized by I/O service at time of use.

	int					abort;
	gsc_dma_ch_t*		dma_channel;		// Use this channel for DMA.
	int					non_blocking;		// Is this non-blocking I/O?

	// Initialized by device specific I/O create code.

	int					bytes_per_sample;	// Sample size in bytes.
	u32					io_reg_offset;		// Offset of device's I/O FIFO.
	VADDR_T				io_reg_vaddr;		// Address of device's I/O FIFO.

	void				(*dev_io_sw_init)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	void				(*dev_io_close)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	void				(*dev_io_open)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	int					(*dev_io_startup)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	long				(*dev_pio_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_bmdma_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_dmdma_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_pio_xfer)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
	long				(*dev_bmdma_xfer)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
	long				(*dev_dmdma_xfer)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);

	struct
	{
		u32				abort;		// ...WAIT_IO_XXX_ABORT
		u32				done;		// ...WAIT_IO_XXX_DONE
		u32				error;		// ...WAIT_IO_XXX_ERROR
		u32				timeout;	// ...WAIT_IO_XXX_TIMEOUT
	} wait;

	// Initialize by GSC commone create code.

	os_mem_t			mem;				// I/O buffer.
	os_sem_t			sem;				// Only one Tx or Rx at a time.
};

struct _dev_data_t
{
	os_pci_t			pci;			// The kernel PCI device descriptor.
	os_data_t			os;				// OS specific data.
	os_spinlock_t		spinlock;		// Control ISR access.
	os_sem_t			sem;			// Control thread access.
	gsc_dev_type_t		board_type;		// Corresponds to basic device type.
	const char*			model;			// Base model number in upper case.
	int					board_index;	// Index of the device being accessed.
	int					users;			// Number of currently active open() requests.
	int					share;			// Were we opened in shared mode?

	gsc_bar_t			bar;			// device register mappings
	gsc_dma_t			dma;			// For DMA based I/O.
	gsc_irq_t			irq;			// For interrut support.
	gsc_wait_node_t*	wait_list;

	struct
	{					// This is for streaming I/O
		dev_io_t		tx;				// Analog Output write
		dev_io_t*		io_streams[DEV_IO_STREAM_QTY];
	} io;

	struct
	{
		VADDR_T			plx_intcsr_32;	// Interrupt Control/Status Register
		VADDR_T			plx_dmaarb_32;	// DMA Arbitration Register
		VADDR_T			plx_dmathr_32;	// DMA Threshold Register

		VADDR_T			gsc_bcr_32;		// Board Control Register
		VADDR_T			gsc_diopr_32;	// Digital I/O Port Register
		VADDR_T			gsc_asiocr_32;	// Auxiliary Sync I/O Control Register
		VADDR_T			gsc_psr_32;		// Primary Status Register
		VADDR_T			gsc_acr_32;		// Assembly Configuration Register
		// We skip the Autocal Values Register as we have no need to access it.
		VADDR_T			gsc_boor_32;	// Buffered Output Operations Register
		VADDR_T			gsc_obtr_32;	// Output Buffer Threshold Register
		VADDR_T			gsc_obsr_32;	// Output Buffer Size Register
		VADDR_T			gsc_aobr_32;	// Output Output Buffer Register
		VADDR_T			gsc_ragr_32;	// Rate-A Generator Register
		VADDR_T			gsc_rbgr_32;	// Rate-B Generator Register
		VADDR_T			gsc_ocr_32;		// Output Configuration Register
	} vaddr;

	struct
	{
		u32				gsc_acr_32;			// Assembly Configuration Register

		int				autocal_ms;			// Maximum ms for autocalibrate

		int				channel_qty;		// The number of channels on the device.
		int				channels_max;		// Maximum channels supported by model.

		int				fgen_max;			// Maximum rate generator rate.
		int				fgen_min;			// Minimum rate generator rate.
		int				fifo_size;			// Size of FIFO - not the fill level.
		int				fref_default;		// Default reference clock rate.
		int				fsamp_max;			// Maximum sample rate.
		int				fsamp_min;			// Minimum sample rate.

		int				initialize_ms;		// Maximum ms for initialize

		int				ndiv_max;			// Maximum Ndiv value.
		int				ndiv_min;			// Minimum Ndiv value.

		int				output_disconnect;	// Is this feature present?
		int				output_filter;		// What is the filter frequency.
		int				output_range;		// What voltage range is supported?

		int				pin_a33;			// What is the function of this pin?

	} cache;
};



// prototypes *****************************************************************

void		dev_io_close(dev_data_t* dev);
int			dev_io_create(dev_data_t* dev);
void		dev_io_destroy(dev_data_t* dev);
int			dev_io_open(dev_data_t* dev);
dev_io_t*	dev_io_write_select(dev_data_t* dev, size_t count);
int			dev_irq_create(dev_data_t* dev);
void		dev_irq_destroy(dev_data_t* dev);
int			dev_write_create(dev_data_t* dev, dev_io_t* io);

int			initialize_ioctl(dev_data_t* dev, void* arg);



#endif
