// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/20AO8C500K/samples/irq/perform.c $
// $Rev: 53736 $
// $Date: 2023-09-21 16:09:42 -0500 (Thu, 21 Sep 2023) $

// 20AO8C500K: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _init(int fd)
{
	int	errs	= 0;

	errs	+= ao8c500k_irq_enable	(fd, -1, 0, AO8C500K_IRQ_INIT_DONE, NULL);
	errs	+= ao8c500k_initialize	(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _irq_init_done_test(int fd)
{
	int	errs	= 0;

	gsc_label("Initialization Done");
	errs	+= ao8c500k_initialize(fd, -1, 0);
	errs	+= irq_wait_test(fd, AO8C500K_WAIT_GSC_INIT_DONE, _init);
	errs	+= ao8c500k_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _autocal(int fd)
{
	int	errs	= 0;

	errs	+= ao8c500k_irq_enable	(fd, -1, 0, AO8C500K_IRQ_AUTOCAL_DONE, NULL);
	errs	+= ao8c500k_autocal		(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _irq_autocal_done_test(int fd)
{
	int	errs	= 0;

	gsc_label("Autocalibrate Done");
	errs	+= ao8c500k_initialize(fd, -1, 0);
	errs	+= irq_wait_test(fd, AO8C500K_WAIT_GSC_AUTOCAL_DONE, _autocal);
	errs	+= ao8c500k_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _dio0_l2h(int fd)
{
	int	errs	= 0;

	errs	+= ao8c500k_dio_dir_out	(fd, -1, 0, AO8C500K_DIO_DIR_OUT_D7_D0,	NULL);
	errs	+= ao8c500k_dio_write	(fd, -1, 0, 0x00,					NULL);
	errs	+= ao8c500k_irq_enable	(fd, -1, 0, AO8C500K_IRQ_DIO0_L2H,		NULL);
	errs	+= ao8c500k_dio_write	(fd, -1, 0, 0x01,					NULL);
	return(errs);
}



//*****************************************************************************
static int _irq_dio0_l2h_test(int fd)
{
	int	errs	= 0;

	gsc_label("DIO 0 Low->High");
	errs	+= ao8c500k_initialize(fd, -1, 0);
	errs	+= irq_wait_test(fd, AO8C500K_WAIT_GSC_DIO0_L2H, _dio0_l2h);
	errs	+= ao8c500k_initialize(fd, -1, 0);
	return(errs);
}


//*****************************************************************************
static int _buf_thr_h2l(int fd)
{
	int	errs	= 0;
	int	i;

	for (i = 0; i < 100; i++)
		errs	+= ao8c500k_reg_write(fd, -1, 0, AO8C500K_GSC_AOBR, 0);

	errs	+= ao8c500k_buf_thresh_level	(fd, -1, 0, 10,						NULL);
	errs	+= ao8c500k_irq_enable		(fd, -1, 0, AO8C500K_IRQ_BUF_THR_H2L,	NULL);
	errs	+= ao8c500k_buf_thresh_level	(fd, -1, 0, 10000,					NULL);
	return(errs);
}




//*****************************************************************************
static int _irq_buf_thr_h2l_test(int fd)
{
	int	errs	= 0;

	gsc_label("Buffer Threshold Hi->Low");
	errs	+= ao8c500k_initialize(fd, -1, 0);
	errs	+= irq_wait_test(fd, AO8C500K_WAIT_GSC_BUF_THR_H2L, _buf_thr_h2l);
	errs	+= ao8c500k_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _buf_thr_l2h(int fd)
{
	int	errs	= 0;
	int	i;

	for (i = 0; i < 100; i++)
		errs	+= ao8c500k_reg_write(fd, -1, 0, AO8C500K_GSC_AOBR, 0);

	errs	+= ao8c500k_buf_thresh_level	(fd, -1, 0, 10000,					NULL);
	errs	+= ao8c500k_irq_enable		(fd, -1, 0, AO8C500K_IRQ_BUF_THR_L2H,	NULL);
	errs	+= ao8c500k_buf_thresh_level	(fd, -1, 0, 10,						NULL);
	return(errs);
}



//*****************************************************************************
static int _irq_buf_thr_l2h_test(int fd)
{
	int	errs	= 0;

	gsc_label("Buffer Threshold Low->Hi");
	errs	+= ao8c500k_initialize(fd, -1, 0);
	errs	+= irq_wait_test(fd, AO8C500K_WAIT_GSC_BUF_THR_L2H, _buf_thr_l2h);
	errs	+= ao8c500k_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _load_ready_h2l(int fd)
{
	int	errs	= 0;

	errs	+= ao8c500k_buf_mode		(fd, -1, 0, AO8C500K_BUF_MODE_OPEN,		NULL);
	errs	+= ao8c500k_irq_enable	(fd, -1, 0, AO8C500K_IRQ_LOAD_READY_H2L,	NULL);
	errs	+= ao8c500k_buf_mode		(fd, -1, 0, AO8C500K_BUF_MODE_CIRC,		NULL);
	return(errs);
}



//*****************************************************************************
static int _irq_load_ready_h2l_test(int fd)
{
	int	errs	= 0;

	gsc_label("Load Ready Hi->Low");
	errs	+= ao8c500k_initialize(fd, -1, 0);
	errs	+= irq_wait_test(fd, AO8C500K_WAIT_GSC_LD_RDY_H2L, _load_ready_h2l);
	errs	+= ao8c500k_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _load_ready_l2h(int fd)
{
	int	errs	= 0;

	errs	+= ao8c500k_buf_mode		(fd, -1, 0, AO8C500K_BUF_MODE_CIRC,		NULL);
	errs	+= ao8c500k_irq_enable	(fd, -1, 0, AO8C500K_IRQ_LOAD_READY_L2H,	NULL);
	errs	+= ao8c500k_buf_mode		(fd, -1, 0, AO8C500K_BUF_MODE_OPEN,		NULL);
	return(errs);
}



//*****************************************************************************
static int _irq_load_ready_l2h_test(int fd)
{
	int	errs	= 0;

	gsc_label("Load Ready Low->Hi");
	errs	+= ao8c500k_initialize(fd, -1, 0);
	errs	+= irq_wait_test(fd, AO8C500K_WAIT_GSC_LD_RDY_L2H, _load_ready_l2h);
	errs	+= ao8c500k_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _burst_ready(int fd)
{
	int	errs	= 0;
	int	i;

	errs	+= ao8c500k_burst_enable	(fd, -1, 0, AO8C500K_BURST_ENABLE_YES,	NULL);

	for (i = 0; i < 100; i++)
		errs	+= ao8c500k_reg_write(fd, -1, 0, AO8C500K_GSC_AOBR, 0);

	errs	+= ao8c500k_irq_enable	(fd, -1, 0, AO8C500K_IRQ_BURST_READY,	NULL);
	errs	+= ao8c500k_clock_enable	(fd, -1, 0, AO8C500K_CLOCK_ENABLE_YES,	NULL);
	return(errs);
}



//*****************************************************************************
static int _irq_burst_ready_test(int fd)
{
	int	errs	= 0;

	gsc_label("Burst Ready");
	errs	+= ao8c500k_initialize(fd, -1, 0);
	errs	+= irq_wait_test(fd, AO8C500K_WAIT_GSC_BURST_READY, _burst_ready);
	errs	+= ao8c500k_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _buf_error(int fd)
{
	int	errs	= 0;

	errs	+= ao8c500k_buf_mode		(fd, -1, 0, AO8C500K_BUF_MODE_CIRC,	NULL);
	errs	+= ao8c500k_irq_enable	(fd, -1, 0, AO8C500K_IRQ_BUF_ERROR,	NULL);
	errs	+= ao8c500k_reg_write	(fd, -1, 0, AO8C500K_GSC_AOBR, 0);
	return(errs);
}



//*****************************************************************************
static int _irq_buf_error_test(int fd)
{
	int	errs	= 0;

	gsc_label("Buffer Error");
	errs	+= ao8c500k_initialize(fd, -1, 0);
	errs	+= irq_wait_test(fd, AO8C500K_WAIT_GSC_BUF_ERROR, _buf_error);
	errs	+= ao8c500k_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int	errs	= 0;

	gsc_label("IRQ");
	printf("\n");
	gsc_label_level_inc();

	errs	+= _irq_init_done_test(args->fd);
	errs	+= _irq_autocal_done_test(args->fd);
	errs	+= _irq_dio0_l2h_test(args->fd);
	errs	+= _irq_buf_thr_h2l_test(args->fd);
	errs	+= _irq_buf_thr_l2h_test(args->fd);
	errs	+= _irq_load_ready_h2l_test(args->fd);
	errs	+= _irq_load_ready_l2h_test(args->fd);
	errs	+= _irq_burst_ready_test(args->fd);
	errs	+= _irq_buf_error_test(args->fd);

	gsc_label_level_dec();
	return(errs);
}


