// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/20AO8C500K/utils/buf_fill_level.c $
// $Rev: 53736 $
// $Date: 2023-09-21 16:09:42 -0500 (Thu, 21 Sep 2023) $

// 20AO8C500K: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ao8c500k_buf_fill_level
*
*	Purpose:
*
*		Provide a visual wrapper for the AO8C500K_IOCTL_BUF_FILL_LEVEL service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao8c500k_buf_fill_level(int fd, int index, int verbose, s32* get)
{
	int			errs;
	const char*	factor	= "";
	int			ret;
	s32			set;

	if (verbose)
		gsc_label_index("Buffer Fill Level", index);

	ret		= ao8c500k_ioctl(fd, AO8C500K_IOCTL_BUF_FILL_LEVEL, &set);
	errs	= ret ? 1 : 0;

	if (get)
		get[0]	= set;

	if (verbose)
	{
		if ((set >= 1024) && ((set % 1024) == 0))
		{
			factor	= "K";
			set		/= 1024;
		}

		printf("%s  (", errs ? "FAIL <---" : "PASS");
		gsc_label_long_comma(set);
		printf("%s samples)\n", factor);
	}

	return(errs);
}



