// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/20AO8C500K/utils/config_ao.c $
// $Rev: 53736 $
// $Date: 2023-09-21 16:09:42 -0500 (Thu, 21 Sep 2023) $

// 20AO8C500K: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ao8c500k_config_ao
*
*	Purpose:
*
*		Configure the given device using common defaults.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fsamp	This is the desired Fsamp rate, of -1 to use the default.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int	ao8c500k_config_ao(int fd, int index, int verbose, s32 fsamp)
{
	s32	chans	= 8;
	int	errs	= 0;
	s32	ndiv;

	if (verbose)
	{
		gsc_label_index("Output Configuration", index);
		printf("\n");
		gsc_label_level_inc();
	}

	errs	+= ao8c500k_initialize			(fd, index, verbose);

	// Settings must be applied after the initialization call!

	errs	+= ao8c500k_tx_io_mode			(fd, index, verbose, AO8C500K_IO_MODE_DEFAULT,			NULL);
	errs	+= ao8c500k_tx_io_over_data		(fd, index, verbose, AO8C500K_TX_IO_OVERFLOW_DEFAULT,	NULL);
	errs	+= ao8c500k_tx_io_over_frame	(fd, index, verbose, AO8C500K_TX_IO_OVERFLOW_DEFAULT,	NULL);
	errs	+= ao8c500k_tx_io_timeout		(fd, index, verbose, AO8C500K_IO_TIMEOUT_DEFAULT,		NULL);

	errs	+= ao8c500k_ao_config			(fd, index, verbose, AO8C500K_AO_CONFIG_SE,				NULL);
	errs	+= ao8c500k_remote_gnd_sense	(fd, index, verbose, AO8C500K_REMOTE_GND_SENSE_NO,		NULL);
	errs	+= ao8c500k_buf_mode			(fd, index, verbose, AO8C500K_BUF_MODE_OPEN,			NULL);
	errs	+= ao8c500k_buf_thresh_level	(fd, index, verbose, 250000L,							NULL);

	errs	+= ao8c500k_query				(fd, index, verbose, AO8C500K_QUERY_CHANNEL_QTY, &chans);
	chans	= (chans == 4) ? 0xF : 0xFF;
	errs	+= ao8c500k_channel_sel			(fd, index, verbose, chans,								NULL);

	errs	+= ao8c500k_data_format			(fd, index, verbose, AO8C500K_DATA_FORMAT_OFF_BIN,		NULL);
	errs	+= ao8c500k_output_mode			(fd, index, verbose, AO8C500K_OUTPUT_MODE_SIMULTANEOUS,	NULL);
	errs	+= ao8c500k_output_filter		(fd, index, verbose, AO8C500K_OUTPUT_FILTER_NO,			NULL);
	errs	+= ao8c500k_output_enable		(fd, index, verbose, AO8C500K_OUTPUT_ENABLE_YES,		NULL);
	errs	+= ao8c500k_range				(fd, index, verbose, AO8C500K_RANGE_5,					NULL);

	errs	+= ao8c500k_fsamp_ao_compute	(fd, index, verbose, fsamp, &ndiv,						NULL);
	errs	+= ao8c500k_gen_a_ndiv			(fd, index, verbose, ndiv,								NULL);
	errs	+= ao8c500k_gen_a_enable		(fd, index, verbose, AO8C500K_GEN_ENABLE_YES,			NULL);
	errs	+= ao8c500k_clock_src			(fd, index, verbose, AO8C500K_CLOCK_SRC_RAG,			NULL);
	errs	+= ao8c500k_clock_enable		(fd, index, verbose, AO8C500K_CLOCK_ENABLE_YES,			NULL);

	errs	+= ao8c500k_xcvr_output			(fd, index, verbose, AO8C500K_XCVR_OUTPUT_YES,			NULL);
	errs	+= ao8c500k_xcvr_pol			(fd, index, verbose, AO8C500K_XCVR_POL_DEFAULT,			NULL);
	errs	+= ao8c500k_xcvr_type			(fd, index, verbose, AO8C500K_XCVR_TYPE_TTL,			NULL);

	errs	+= ao8c500k_burst_enable		(fd, index, verbose, AO8C500K_BURST_ENABLE_NO,			NULL);
	errs	+= ao8c500k_burst_trig_src		(fd, index, verbose, AO8C500K_BURST_TRIG_SRC_RBG,		NULL);
	errs	+= ao8c500k_gen_b_enable		(fd, index, verbose, AO8C500K_GEN_ENABLE_NO,			NULL);
	errs	+= ao8c500k_gen_b_ndiv			(fd, index, verbose, 0xFFFF,							NULL);

	errs	+= ao8c500k_fsamp_ao_report_all	(fd, index, verbose,									NULL);

	// Settings affecting the sample rate or voltage range must generally
	// be applied before autocalibration.
	// Refer to the board user manual for clarification.
	// If the board supports teh Output Disconnect feature, then uncomment
	// this call.
//	errs	+= ao8c500k_autocal				(fd, index, verbose);	// cable may need disconnecting for autocal

	// Interrupt selections should be applied after autocalibration as the
	// driver overwrites the current selection to detect the Autocalibration
	// Done interrupt. If an application intends to detect the Autocalibration
	// Done interrupt, then the interrupt selection may be made beforehand.
	errs	+= ao8c500k_irq_enable			(fd, index, verbose, 0,									NULL);

	// The final step is to clear the buffer. This service clears the FIFO and
	// resets the data overflow and frame overflow status bits.
	errs	+= ao8c500k_buf_clear			(fd, index, verbose);

	if (verbose)
		gsc_label_level_dec();

	return(errs);
}



