// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/20AO8C500K/utils/query.c $
// $Rev: 53736 $
// $Date: 2023-09-21 16:09:42 -0500 (Thu, 21 Sep 2023) $

// 20AO8C500K: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ao8c500k_query
*
*	Purpose:
*
*		Provide a visual wrapper for the AO8C500K_IOCTL_QUERY service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the query option to access.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao8c500k_query(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs	= 0;
	const char*	ptr;
	s32			query	= set;
	int			ret;
	s32			tmp;

	switch (query)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Query Error (#%ld)", (long) query);
			break;

		case AO8C500K_QUERY_AUTOCAL_MS:			ptr	= "Autocal Duration";	break;
		case AO8C500K_QUERY_CHANNEL_MAX:		ptr	= "Max Channels";		break;
		case AO8C500K_QUERY_CHANNEL_QTY:		ptr	= "Installed Channels";	break;
		case AO8C500K_QUERY_COUNT:				ptr	= "Query Options";		break;
		case AO8C500K_QUERY_DEVICE_TYPE:		ptr	= "Device Type";		break;
		case AO8C500K_QUERY_FGEN_MAX:			ptr	= "Fgen Max";			break;
		case AO8C500K_QUERY_FGEN_MIN:			ptr	= "Fgen Min";			break;
		case AO8C500K_QUERY_FIFO_SIZE:			ptr	= "FIFO Size";			break;
		case AO8C500K_QUERY_FREF_DEFAULT:		ptr	= "Fref Default";		break;
		case AO8C500K_QUERY_FSAMP_MAX:			ptr	= "Fsamp Max";			break;
		case AO8C500K_QUERY_FSAMP_MIN:			ptr	= "Fsamp Min";			break;
		case AO8C500K_QUERY_INIT_MS:			ptr	= "Init. Duration";		break;
		case AO8C500K_QUERY_NDIV_MAX:			ptr	= "Ndiv Max";			break;
		case AO8C500K_QUERY_NDIV_MIN:			ptr	= "Ndiv Min";			break;
		case AO8C500K_QUERY_OUTPUT_DISCONNECT:	ptr	= "Output Disconnect";	break;
		case AO8C500K_QUERY_OUTPUT_FILTER:		ptr	= "Output Filter";		break;
		case AO8C500K_QUERY_OUTPUT_RANGE:		ptr	= "Output Range";		break;
		case AO8C500K_QUERY_PIN_A33:			ptr	= "Pin A33 Function";	break;
	}

	if (verbose)
		gsc_label_index(ptr, index);

	ret		= ao8c500k_ioctl(fd, AO8C500K_IOCTL_QUERY, &set);
	errs	+= ret ? 1 : 0;

	switch (query)
	{
		default:

			errs++;
			sprintf(buf, "%ld", (long) set);
			break;

		case AO8C500K_QUERY_CHANNEL_MAX:
		case AO8C500K_QUERY_CHANNEL_QTY:
		case AO8C500K_QUERY_COUNT:
		case AO8C500K_QUERY_NDIV_MAX:
		case AO8C500K_QUERY_NDIV_MIN:

			gsc_label_long_comma_buf(set, buf);
			break;

		case AO8C500K_QUERY_FGEN_MAX:
		case AO8C500K_QUERY_FGEN_MIN:
		case AO8C500K_QUERY_FREF_DEFAULT:
		case AO8C500K_QUERY_OUTPUT_FILTER:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " Hz");
			break;

		case AO8C500K_QUERY_AUTOCAL_MS:
		case AO8C500K_QUERY_INIT_MS:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " ms");
			break;

		case AO8C500K_QUERY_FSAMP_MAX:
		case AO8C500K_QUERY_FSAMP_MIN:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " S/S");
			break;

		case AO8C500K_QUERY_FIFO_SIZE:

			ptr	= "";
			tmp	= set;

			if ((tmp) && ((tmp % 1024) == 0))
			{
				ptr	= "K";
				tmp	/= 1024;
			}

			gsc_label_long_comma_buf(tmp, buf);
			strcat(buf, ptr);
			strcat(buf, " Words");
			break;

		case AO8C500K_QUERY_DEVICE_TYPE:

			if (set == GSC_DEV_TYPE_20AO8C500K)
			{
				strcpy(buf, "20AO8C500K");
			}
			else
			{
				errs++;
				sprintf(buf, "INVALID: %ld", (long) set);
			}

			break;


		case AO8C500K_QUERY_OUTPUT_DISCONNECT:

			sprintf(buf, "%s", set ? "Supported" : "Not Supported");
			break;

		case AO8C500K_QUERY_OUTPUT_RANGE:

			switch (set)
			{
				default:

					errs++;
					sprintf(buf, "INVALID: %ld", (long) set);
					break;

				case AO8C500K_QUERY_OUTPUT_RANGE_10:

					strcpy(buf, "+-10V, +-5V");
					break;

				case AO8C500K_QUERY_OUTPUT_RANGE_5:

					strcpy(buf, "+-5V, +-2.5V");
					break;
			}

			break;

		case AO8C500K_QUERY_PIN_A33:

			switch (set)
			{
				default:

					errs++;
					sprintf(buf, "INVALID: %ld", (long) set);
					break;

				case AO8C500K_QUERY_PIN_A33_OUT_RTN:

					strcpy(buf, "Output Return");
					break;

				case AO8C500K_QUERY_PIN_A33_RMT_GND_S:

					strcpy(buf, "Remote Ground Sense");
					break;
			}

			break;
	}

	if (verbose)
	{
		if (errs)
			printf("FAIL <---  (%s)\n", buf);
		else
			printf("%s\n", buf);
	}

	if (get)
		get[0]	= set;

	return(errs);
}


