// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/20AO8C500K/utils/range.c $
// $Rev: 53736 $
// $Date: 2023-09-21 16:09:42 -0500 (Thu, 21 Sep 2023) $

// 20AO8C500K: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ao8c500k_range
*
*	Purpose:
*
*		Provide a visual wrapper for the AO8C500K_IOCTL_RANGE service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao8c500k_range(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("Voltage Range", index);

	strcpy(buf, "Range is not supported: ");

	switch (set)
	{
		default:					strcat(buf, "INVALID");		break;
		case -1:					strcat(buf, "query");		break;
		case AO8C500K_RANGE_2_5:	strcat(buf, "+-2.5 Volts");	break;
		case AO8C500K_RANGE_5:		strcat(buf, "+-5 Volts");	break;
		case AO8C500K_RANGE_10:		strcat(buf, "+-10 Volts");	break;
	}

	ret		= ao8c500k_ioctl(fd, AO8C500K_IOCTL_RANGE, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case -1:

			ptr	= buf;
			break;

		case AO8C500K_RANGE_2_5:

			ptr	= "+-2.5 Volts";
			break;

		case AO8C500K_RANGE_5:

			ptr	= "+-5 Volts";
			break;

		case AO8C500K_RANGE_10:

			ptr	= "+-10 Volts";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


