// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/20AO8C500K/utils/remote_gnd_sense.c $
// $Rev: 53736 $
// $Date: 2023-09-21 16:09:42 -0500 (Thu, 21 Sep 2023) $

// 20AO8C500K: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ao8c500k_remote_gnd_sense
*
*	Purpose:
*
*		Provide a visual wrapper for the AO8C500K_IOCTL_REMOTE_GND_SENSE service.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the board to access. Ignore if < 0.
*
*		verbose	0: errors only, !0: all output
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao8c500k_remote_gnd_sense(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	s32			ret;

	if (verbose)
		gsc_label_index("Remote Ground Sense", index);

	ret		= ao8c500k_ioctl(fd, AO8C500K_IOCTL_REMOTE_GND_SENSE, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs	= 1;
			ptr		= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case -1:

			ptr	= "This service is not supported by this board.";
			break;

		case AO8C500K_REMOTE_GND_SENSE_NO:

			ptr	= "No, Disabled";
			break;

		case AO8C500K_REMOTE_GND_SENSE_YES:

			ptr	= "Yes, Enabled";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


